/* Terraform - (C) 1997-1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogRotate.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogRotate::TFDialogRotate (HeightField *HF, HeightFieldDraw *HFD)
	        : TFPreviewDialog (HF, HFD, "Terraform Rotate Dialog", 
			"Rotation Parameters", 6, 4),
		d_optionsFrame ("Rotate clockwise by"),
		d_rotateTable (6, 6, FALSE),
		d_rLabel1 ("0"),
		d_rLabel2 ("90"),
		d_rLabel3 ("180"),
		d_rLabel4 ("270"),
		d_unitLabel ("degrees")
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogRotate\n");
	d_RB1 = d_RB2 = d_RB3 = d_RB4 = NULL;
	buildDialogWindow ();
        this->setHFobjs (HF, HFD);
	sprintf (this->d_winTitle, "Rotate: %s", d_HF->getName());
	this->set_title (this->d_winTitle);
        this->iterateEvents ();
        updatePreviewCallback ();

	connect_to_method (d_RB1->clicked, this, &TFDialogRotate::updatePreviewCallback);
	connect_to_method (d_RB2->clicked, this, &TFDialogRotate::updatePreviewCallback);
	connect_to_method (d_RB3->clicked, this, &TFDialogRotate::updatePreviewCallback);
	connect_to_method (d_RB4->clicked, this, &TFDialogRotate::updatePreviewCallback);
}


/*
 *  destructor: clean up 
 */
TFDialogRotate::~TFDialogRotate ()
{
	if (d_RB1) delete d_RB1;
	if (d_RB2) delete d_RB2;
	if (d_RB3) delete d_RB3;
	if (d_RB4) delete d_RB4;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogRotate\n");
}


/*
 *  insertOptions: fill the dialog's VBox
 */
void TFDialogRotate::insertOptions ()
{
	GSList		*radioGroup=NULL;

	// parameter settings  
	d_optionsFrame.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_optionsFrame.set_border_width (6);

	// table for d_RBs 
	d_rotateTable.set_row_spacings (8);
	d_rotateTable.set_col_spacings (1);
	d_rotateTable.set_border_width (6);
	d_optionsFrame.add (d_rotateTable);

	// place the buttons 
	d_rotateTable.attach (d_rLabel1, 2, 3 , 0, 1, 0, 0, 0, 0);
	d_rLabel1.show ();
	d_RB1 = new Gtk_RadioButton (radioGroup);
	radioGroup = d_RB1->group();
	d_rotateTable.attach (*d_RB1, 2, 3, 1, 2, 0, 0, 0, 0);
	d_RB1->show ();

	d_rotateTable.attach (d_rLabel2, 4, 5, 2, 3, 0, 0, 0, 0);
	d_rLabel2.show ();
	d_RB2 = new Gtk_RadioButton (radioGroup);
	radioGroup = d_RB2->group();
	d_rotateTable.attach (*d_RB2, 3, 4, 2, 3, 0, 0, 0, 0);
	d_RB2->show ();

	d_rotateTable.attach (d_rLabel3, 2, 3, 4, 5, 0, 0, 0, 0);
	d_rLabel3.show ();
	d_RB3 = new Gtk_RadioButton (radioGroup);
	radioGroup = d_RB3->group();
	d_rotateTable.attach (*d_RB3, 2, 3, 3, 4, 0, 0, 0, 0);
	d_RB3->set_active (TRUE);
	d_RB3->show ();

	d_rotateTable.attach (d_rLabel4, 0, 1, 2, 3, 0, 0, 0, 0);
	d_rLabel4.show ();
	d_RB4 = new Gtk_RadioButton (radioGroup);
	radioGroup = d_RB4->group();
	d_rotateTable.attach (*d_RB4, 1, 2, 2, 3, 0, 0, 0, 0);
	d_RB4->show ();

	// place unit label
	d_rotateTable.attach (d_unitLabel, 5, 6, 5, 6, 0, 0, 0, 0);
	d_unitLabel.show ();

	this->d_table->attach (d_optionsFrame, 3, 6, 1, 3);
	d_rotateTable.show();
	d_optionsFrame.show ();
	this->d_baseFrame->show ();
        this->get_vbox()->show ();
}


/*
 *  buildDialogWindow:
 */
void TFDialogRotate::buildDialogWindow ()
{
	this->set_usize (350, 225);
	this->set_title (this->d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	this->insertPreview ();
	insertOptions ();
	this->fillActionArea ();

	this->show ();
}


/*
 *  buttonCallbackApply: rotate the Height Field
 */
void TFDialogRotate::buttonCallbackApply ()
{
	int		value=180; 

	if (d_RB1->get_active())
		value = 0;
	else
	if (d_RB2->get_active())
		value = 270;		// Rotate90 goes counterclockwise
	else
	if (d_RB3->get_active())
		value = 180;
	else
	if (d_RB4->get_active())
		value = 90;		// Rotate90 goes counterclockwise
	else
		SanityCheck::bailout ((TRUE), "Button out of range", "TFDialogRotate::buttonCallbackApply");

	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogRotate::buttonCallbackApply");

	if (value)
		{
		d_HFO->rotate90 (value);
		d_HFD->draw ();

		// only do this if window stays open 
		if (d_applyHit)
			{
			this->setHFobjs (d_HF, d_HFD);
			updatePreviewCallback ();
			}
		}
	this->d_applyHit = TRUE;
}


/*
 *  updatePreviewCallback: update the preview after a widget has been updated.
 */
void TFDialogRotate::updatePreviewCallback ()
{
	if (!d_cbUsePreview.get_active())
		return;

	int	value = 180;

	SanityCheck::bailout ((!d_HFPreview), "d_HFPreview==NULL", "TFDialogFill");
	SanityCheck::bailout ((!d_HFOPreview), "d_HFOPreview==NULL", "TFDialogFill");
	SanityCheck::bailout ((!d_HFDPreview), "d_HFDPreview==NULL", "TFDialogFill");
	this->previewUpToDate ();
	d_HFPreview->restore ();
	if (d_RB1->get_active())
		value = 0;
	else
	if (d_RB2->get_active())
		value = 270;		// Rotate90 goes counterclockwise
	else
	if (d_RB3->get_active())
		value = 180;
	else
	if (d_RB4->get_active())
		value = 90;		// Rotate90 goes counterclockwise
	else
		SanityCheck::bailout ((TRUE), "Button out of range", "TFDialogRotate::buttonCallbackApply");
	d_HFOPreview->rotate90 (value);
	d_HFDPreview->setColormap (d_HFD->getColormap());
	d_HFDPreview->draw ();
}

